// Copyright 2009 Christopher Smith
// 
// This file is part of smithers.calculator.
// 
// smithers.calculator is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by the
// Free Software Foundation, either version 3 of the License, or (at your
// option) any later version.
// 
// smithers.calculator is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
// more details.
// 
// You should have received a copy of the GNU General Public License along with
// smithers.calculator. If not, see <http://www.gnu.org/licenses/>.

package smithers.calculator;

class MethodsAnalytic {
    
    static CalcObject exp(CalcObject... args) {
        if (args[0] instanceof CalcDouble) {
            return new CalcDouble(Math.exp(((CalcDouble)args[0]).value));
        } else if (args[0] instanceof CalcComplexDouble) {
            double x = ((CalcComplexDouble)args[0]).re;
            double y = ((CalcComplexDouble)args[0]).im;
            return CalcComplexDouble.fromAbsArg(Math.exp(x), y);
        } else {
            throw new RuntimeException("Not a complex or double value");
        }
    }
    
    static CalcObject log(CalcObject... args) {
        if (args[0] instanceof CalcDouble) {
            return new CalcDouble(Math.log(((CalcDouble)args[0]).value));
        } else if (args[0] instanceof CalcComplexDouble) {
            double re = Math.log(((CalcComplexDouble)args[0]).abs());
            double im = ((CalcComplexDouble)args[0]).arg();
            return new CalcComplexDouble(re, im);
        } else {
            throw new RuntimeException("Not a complex or double value");
        }
    }
    
    static CalcObject sin(CalcObject... args) {
        if (args[0] instanceof CalcDouble) {
            return new CalcDouble(Math.sin(((CalcDouble)args[0]).value));
        } else if (args[0] instanceof CalcComplexDouble) {
            double x = ((CalcComplexDouble)args[0]).re;
            double y = ((CalcComplexDouble)args[0]).im;
            double re = Math.sin(x) * Math.cosh(y);
            double im = Math.cos(x) * Math.sinh(y);
            return new CalcComplexDouble(re, im);
        } else {
            throw new RuntimeException("Not a complex or double value");
        }
    }
    
    static CalcObject cos(CalcObject... args) {
        if (args[0] instanceof CalcDouble) {
            return new CalcDouble(Math.cos(((CalcDouble)args[0]).value));
        } else if (args[0] instanceof CalcComplexDouble) {
            double x = ((CalcComplexDouble)args[0]).re;
            double y = ((CalcComplexDouble)args[0]).im;
            double re = Math.cos(x) * Math.cosh(y);
            double im = -Math.sin(x) * Math.sinh(y);
            return new CalcComplexDouble(re, im);
        } else {
            throw new RuntimeException("Not a complex or double value");
        }
    }
    
    static CalcObject tan(CalcObject... args) {
        if (args[0] instanceof CalcDouble) {
            return new CalcDouble(Math.tan(((CalcDouble)args[0]).value));
        } else if (args[0] instanceof CalcComplexDouble) {
            double x = ((CalcComplexDouble)args[0]).re;
            double y = ((CalcComplexDouble)args[0]).im;
            double denom = Math.cos(2*x) + Math.cosh(2*y);
            double re = Math.sin(2*x);
            double im = Math.sinh(2*y);
            return new CalcComplexDouble(re/denom, im/denom);
        } else {
            throw new RuntimeException("Not a complex or double value");
        }
    }
    
}
