// Copyright 2010 Christopher Smith
// 
// This file is part of smithers.logicpuzzles.
// 
// smithers.logicpuzzles is free software: you can redistribute it and/or
// modify it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or (at your
// option) any later version.
// 
// smithers.logicpuzzles is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
// more details.
// 
// You should have received a copy of the GNU General Public License along with
// smithers.logicpuzzles. If not, see <http://www.gnu.org/licenses/>.

package smithers.logicpuzzles;

/**
 * Interface representing a "state" of a logic puzzle - that is, some stage
 * along an attempt at solving it.
 */
public interface PuzzleState<P extends Puzzle> {
    
    /**
     * Returns the puzzle which this is a state of.
     * 
     * @return the {@code Puzzle} which this {@code PuzzleState} relates to.
     */
    public P getPuzzle();
    
    /**
     * Determines whether or not the puzzle has been completely solved.
     * 
     * @return {@code true} iff the puzzle has been completely solved
     */
    public boolean isSolved();
    
    /**
     * Generates a {@code String} representing the current state of the puzzle.
     * May or may not include some information about the puzzle itself.
     * 
     * @return a {@code String} representing the state
     */
    public String toString();
    
}
