// Copyright 2009, 2010 Christopher Smith
// 
// This file is part of smithers.twisty.
// 
// smithers.twisty is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
// 
// smithers.twisty is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
// more details.
// 
// You should have received a copy of the GNU General Public License along with
// smithers.twisty. If not, see <http://www.gnu.org/licenses/>.

package smithers.twisty;

/**
 * Represents a notation for describing sequences of twists in twisty puzzle
 * such as Singmaster Notation for the Rubik's Cube.
 */
public interface PuzzleNotation<P extends TwistyPuzzle> {
    
    /**
     * Gets the sequence of twists determined by the description. This should
     * return an array of instances of classes which will be accepted (will not
     * throw {@code ClassCastException}) by the implementation of {@code
     * TwistyPuzzle.twist(Twist)} in the class {@code P}, although they may not
     * be possible from all positions of the puzzle (or even the current
     * position of the puzzle argument).
     * 
     * @param description the twists in this notation
     * @param puzzle      a prototype puzzle to use in case the puzzle type
     *                    itself has parameters necessary for interpreting the
     *                    notation
     * @return the twists specified
     */
    public Twist[] getTwists(String description, P puzzle);
    
}
