// Copyright 2009, 2010, 2012 Christopher Smith
// 
// This file is part of smithers.util.
// 
// smithers.util is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
// 
// smithers.util is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
// more details.
// 
// You should have received a copy of the GNU General Public License along with
// smithers.util. If not, see <http://www.gnu.org/licenses/>.

package smithers.util;

/**
 * Implementation of a disjoint-set data structure and union-find algorithm.
 * This class represents an individual node in a disjoint-set forest. The
 * forest is optimised by path compression on each method call.
 */
public class UnionFindNode {
    
    private UnionFindNode parent;
    
    /**
     * Creates a new node in a set of its own.
     */
    public UnionFindNode() {
        parent = this;
    }
    
    private UnionFindNode top() {
        if (parent == this) return this;
        return parent = parent.top();
    }
    
    /**
     * Tests whether this node and the given node are in the same set.
     * 
     * @param node the node to test against this one
     * @return {@code true} iff both nodes are in the same set
     */
    public boolean sameSet(UnionFindNode node) {
        return this.top() == node.top();
    }
    
    /**
     * Merges the set containing this node and the set containing the given
     * node.
     * 
     * @param node a node in the set to merge with
     */
    public void union(UnionFindNode node) {
        node.top().parent = this.top();
    }
    
}
